/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __DUNGEON_ROOM_STATE_MODEL_H__
#define __DUNGEON_ROOM_STATE_MODEL_H__

typedef struct DungeonRoomStateModel DungeonRoomStateModel;

#define DUNGEON_ROOM_STATE_EVENT_OBJECT_ADDED "DUNGEON_ROOM_STATE_EVENT_OBJECT_ADDED"
#define DUNGEON_ROOM_STATE_EVENT_OBJECT_REMOVED "DUNGEON_ROOM_STATE_EVENT_OBJECT_REMOVED"

#include "convergence/DungeonRoomModel.h"
#include "convergence/GameObjectModel.h"
#include "stemobject/StemObject.h"
#include "utilities/Atom.h"
#include "utilities/EventDispatcher.h"

#define DungeonRoomStateModel_structContents \
	StemObject_structContents \
	\
	EventDispatcher * eventDispatcher; \
	Atom name; \
	unsigned int tileCountX; \
	unsigned int tileCountY; \
	struct DungeonTile * tiles; \
	char * tileCanvasFileName; \
	size_t objectCount; \
	size_t objectListSize; \
	GameObjectModel ** objects; \
	\
	compat_type(DungeonRoomStateModel *) (* copy)(compat_type(DungeonRoomStateModel *) self); \
	void (* addObject)(compat_type(DungeonRoomStateModel *) self, compat_type(GameObjectModel *) object); \
	void (* removeObject)(compat_type(DungeonRoomStateModel *) self, compat_type(GameObjectModel *) object, bool dispose); \
	void (* removeObjectAtIndex)(compat_type(DungeonRoomStateModel *) self, size_t objectIndex, bool dispose); \
	void (* softReset)(compat_type(DungeonRoomStateModel *) self);

struct DungeonRoomStateModel {
	DungeonRoomStateModel_structContents
};

DungeonRoomStateModel * DungeonRoomStateModel_create(Atom name, unsigned int tileCountX, unsigned int tileCountY, struct DungeonTile * tiles, const char * tileCanvasFileName);
void DungeonRoomStateModel_init(compat_type(DungeonRoomStateModel *) selfPtr, Atom name, unsigned int tileCountX, unsigned int tileCountY, struct DungeonTile * tiles, const char * tileCanvasFileName);
void DungeonRoomStateModel_initCopy(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(DungeonRoomStateModel *) originalPtr);
void DungeonRoomStateModel_dispose(compat_type(DungeonRoomStateModel *) selfPtr);

compat_type(DungeonRoomStateModel *) DungeonRoomStateModel_copy(compat_type(DungeonRoomStateModel *) selfPtr);
void DungeonRoomStateModel_addObject(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(GameObjectModel *) object);
void DungeonRoomStateModel_removeObject(compat_type(DungeonRoomStateModel *) selfPtr, compat_type(GameObjectModel *) object, bool dispose);
void DungeonRoomStateModel_removeObjectAtIndex(compat_type(DungeonRoomStateModel *) selfPtr, size_t objectIndex, bool dispose);
void DungeonRoomStateModel_softReset(compat_type(DungeonRoomStateModel *) selfPtr);

#endif
