/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __DUNGEON_ROOM_VIEW_H__
#define __DUNGEON_ROOM_VIEW_H__

typedef struct DungeonRoomView DungeonRoomView;

#define TILE_SIZE 1.0f

#include "3dmath/Matrix.h"
#include "convergence/DungeonRoomStateModel.h"
#include "convergence/TextureCanvas.h"
#include "glgraphics/GLIncludes.h"
#include "gltexture/GLTexture.h"
#include "resourcemanager/ResourceManager.h"
#include "stemobject/StemObject.h"

#define DungeonRoomView_structContents \
	StemObject_structContents \
	\
	DungeonRoomStateModel * stateModel; \
	ResourceManager * resourceManager; \
	GLushort numVertices; \
	GLushort numIndexes; \
	GLuint vboID; \
	GLuint eboID; \
	TextureCanvas * tileTextureCanvas; \
	GLTexture * tileTexture; \
	\
	void (* draw)(compat_type(DungeonRoomView *) self, Matrix modelviewMatrix);

struct DungeonRoomView {
	DungeonRoomView_structContents
};

DungeonRoomView * DungeonRoomView_create(compat_type(DungeonRoomStateModel *) stateModel, ResourceManager * resourceManager);
void DungeonRoomView_init(compat_type(DungeonRoomView *) selfPtr, compat_type(DungeonRoomStateModel *) stateModel, ResourceManager * resourceManager);
void DungeonRoomView_dispose(compat_type(DungeonRoomView *) selfPtr);

void DungeonRoomView_draw(compat_type(DungeonRoomView *) selfPtr, Matrix modelviewMatrix);

#endif
