/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/GameObjectModel.h"

void GameObjectModel_init(compat_type(GameObjectModel *) selfPtr, DungeonRoomStateModel * stateModel) {
	GameObjectModel * self = selfPtr;
	
	StemObject_init(self);
	self->dispose = GameObjectModel_dispose;
	self->update = GameObjectModel_update;
	self->softReset = GameObjectModel_softReset;
	self->atRest = GameObjectModel_atRest;
	self->roughBounds = NULL;
	self->collisionGeometry = NULL;
	self->wouldCollide = GameObjectModel_wouldCollide;
	self->collision = GameObjectModel_collision;
	self->interpolate = GameObjectModel_interpolate;
	
	self->eventDispatcher = EventDispatcher_create(self);
	self->stateModel = stateModel;
	self->type = GAME_OBJECT_TYPE_GENERIC;
	self->markedForRemoval = false;
}

void GameObjectModel_initCopy(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) originalPtr) {
	GameObjectModel * self = selfPtr, * original = originalPtr;
	
	GameObjectModel_init(self, original->stateModel);
	self->markedForRemoval = original->markedForRemoval;
}

void GameObjectModel_dispose(compat_type(GameObjectModel *) selfPtr) {
	GameObjectModel * self = selfPtr;
	
	self->eventDispatcher->dispose(self->eventDispatcher);
	StemObject_dispose(selfPtr);
}

void GameObjectModel_update(compat_type(GameObjectModel *) selfPtr) {
}

void GameObjectModel_softReset(compat_type(GameObjectModel *) selfPtr) {
}

bool GameObjectModel_atRest(compat_type(GameObjectModel *) selfPtr) {
	return true;
}

bool GameObjectModel_wouldCollide(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) object) {
	return true;
}

void GameObjectModel_collision(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time) {
}

void GameObjectModel_interpolate(compat_type(GameObjectModel *) selfPtr, float time) {
}
