/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __GAME_OBJECT_MODEL_H__
#define __GAME_OBJECT_MODEL_H__

typedef struct GameObjectModel GameObjectModel;

typedef enum GameObjectType {
	GAME_OBJECT_TYPE_GENERIC,
	GAME_OBJECT_TYPE_PLAYER,
	GAME_OBJECT_TYPE_ENEMY,
	GAME_OBJECT_TYPE_SWORD_SWING,
	GAME_OBJECT_TYPE_PROJECTILE,
	GAME_OBJECT_TYPE_DOOR,
	GAME_OBJECT_TYPE_HEAL_POWERUP,
	GAME_OBJECT_TYPE_KEY,
	GAME_OBJECT_TYPE_RANK_UP_POWERUP,
	GAME_OBJECT_TYPE_STAIRCASE,
	GAME_OBJECT_TYPE_ROOM_EXIT
} GameObjectType;

#include "convergence/CollisionManager.h"
#include "convergence/DungeonRoomStateModel.h"
#include "stemobject/StemObject.h"
#include "utilities/EventDispatcher.h"

#define GameObjectModel_structContents \
	StemObject_structContents \
	\
	EventDispatcher * eventDispatcher; \
	DungeonRoomStateModel * stateModel; \
	GameObjectType type; \
	bool markedForRemoval; \
	\
	compat_type(GameObjectModel *) (* copy)(compat_type(GameObjectModel *) self); \
	void (* update)(compat_type(GameObjectModel *) self); \
	void (* softReset)(compat_type(GameObjectModel *) self); \
	bool (* atRest)(compat_type(GameObjectModel *) self); \
	struct rect4f (* roughBounds)(compat_type(GameObjectModel *) self); \
	struct collisionGeometry (* collisionGeometry)(compat_type(GameObjectModel *) self); \
	bool (* wouldCollide)(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) object); \
	void (* collision)(compat_type(GameObjectModel *) self, compat_type(GameObjectModel *) object, const struct collision collision, float time); \
	void (* interpolate)(compat_type(GameObjectModel *) self, float time);

struct GameObjectModel {
	GameObjectModel_structContents
};

void GameObjectModel_init(compat_type(GameObjectModel *) selfPtr, DungeonRoomStateModel * stateModel);
void GameObjectModel_initCopy(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) originalPtr);
void GameObjectModel_dispose(compat_type(GameObjectModel *) selfPtr);

void GameObjectModel_update(compat_type(GameObjectModel *) selfPtr);
void GameObjectModel_softReset(compat_type(GameObjectModel *) selfPtr);
bool GameObjectModel_atRest(compat_type(GameObjectModel *) selfPtr);
bool GameObjectModel_wouldCollide(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);
void GameObjectModel_collision(compat_type(GameObjectModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time);
void GameObjectModel_interpolate(compat_type(GameObjectModel *) selfPtr, float time);

#endif
