/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/CollisionManager.h"
#include "convergence/GameStateController.h"

GameStateController * GameStateController_create(GameStateModel * model) {
	stemobject_create_implementation(GameStateController, init, model)
}

void GameStateController_init(compat_type(GameStateController *) selfPtr, GameStateModel * model) {
	GameStateController * self = selfPtr;
	
	StemObject_init(self);
	self->dispose = GameStateController_dispose;
	self->update = GameStateController_update;
	self->atRest = GameStateController_atRest;
	
	self->model = model;
}

void GameStateController_dispose(compat_type(GameStateController *) selfPtr) {
	GameStateController * self = selfPtr;
	
	StemObject_dispose(self);
}

void GameStateController_update(compat_type(GameStateController *) selfPtr) {
	GameStateController * self = selfPtr;
	size_t objectIndex;
	
	for (objectIndex = 0; objectIndex < self->model->activeRoom->objectCount; objectIndex++) {
		self->model->activeRoom->objects[objectIndex]->update(self->model->activeRoom->objects[objectIndex]);
	}
	
	CollisionManager_run(self->model->activeRoom);
	
	objectIndex = self->model->activeRoom->objectCount;
	do {
		objectIndex--;
		if (self->model->activeRoom->objects[objectIndex]->markedForRemoval) {
			self->model->activeRoom->removeObjectAtIndex(self->model->activeRoom, objectIndex, true);
		}
	} while (objectIndex > 0);
	
	if (self->model->exitName != NULL) {
		self->model->exit(self->model);
	}
}

bool GameStateController_atRest(compat_type(GameStateController *) selfPtr) {
	GameStateController * self = selfPtr;
	size_t objectIndex;
	
	for (objectIndex = 0; objectIndex < self->model->activeRoom->objectCount; objectIndex++) {
		if (!self->model->activeRoom->objects[objectIndex]->atRest(self->model->activeRoom->objects[objectIndex])) {
			return false;
		}
	}
	return true;
}
