/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/HealPowerupModel.h"

HealPowerupModel * HealPowerupModel_create(DungeonRoomStateModel * stateModel, int rank, Vector2 position) {
	stemobject_create_implementation(HealPowerupModel, init, stateModel, rank, position)
}

static void sharedInit(HealPowerupModel * self) {
	self->type = GAME_OBJECT_TYPE_HEAL_POWERUP;
	self->dispose = HealPowerupModel_dispose;
	self->copy = HealPowerupModel_copy;
	self->roughBounds = HealPowerupModel_roughBounds;
	self->collisionGeometry = HealPowerupModel_collisionGeometry;
	self->wouldCollide = HealPowerupModel_wouldCollide;
	self->collision = HealPowerupModel_collision;
}

void HealPowerupModel_init(compat_type(HealPowerupModel *) selfPtr, DungeonRoomStateModel * stateModel, int rank, Vector2 position) {
	HealPowerupModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->rank = rank;
	switch (rank) {
		case 0:
		default:
			self->amount = 5;
			self->radius = 0.1875f;
			break;
		case 1:
			self->amount = 10;
			self->radius = 0.28125f;
			break;
		case 2:
			self->amount = 20;
			self->radius = 0.34375f;
			break;
	}
	self->position = position;
}

void HealPowerupModel_initCopy(compat_type(HealPowerupModel *) selfPtr, compat_type(HealPowerupModel *) originalPtr) {
	HealPowerupModel * self = selfPtr;
	HealPowerupModel * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->rank = original->rank;
	self->amount = original->amount;
	self->radius = original->radius;
	self->position = original->position;
}

void HealPowerupModel_dispose(compat_type(HealPowerupModel *) selfPtr) {
	HealPowerupModel * self = selfPtr;
	
	StemObject_dispose(self);
}

compat_type(HealPowerupModel *) HealPowerupModel_copy(compat_type(HealPowerupModel *) selfPtr) {
	stemobject_copy_implementation(HealPowerupModel, initCopy)
}

struct rect4f HealPowerupModel_roughBounds(compat_type(HealPowerupModel *) selfPtr) {
	HealPowerupModel * self = selfPtr;
	struct rect4f bounds;
	
	bounds.left = self->position.x - self->radius;
	bounds.right = self->position.x + self->radius;
	bounds.bottom = self->position.y - self->radius;
	bounds.top = self->position.y + self->radius;
	return bounds;
}

struct collisionGeometry HealPowerupModel_collisionGeometry(compat_type(HealPowerupModel *) selfPtr) {
	HealPowerupModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->position.x - self->radius;
	geometry.u.sweptAABB.current.y = self->position.y - self->radius;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = self->radius * 2.0f;
	geometry.u.sweptAABB.height = self->radius * 2.0f;
	return geometry;
}

bool HealPowerupModel_wouldCollide(compat_type(HealPowerupModel *) selfPtr, compat_type(GameObjectModel *) objectPtr) {
	HealPowerupModel * self = selfPtr;
	GameObjectModel * object = objectPtr;
	
	switch (object->type) {
		case GAME_OBJECT_TYPE_PLAYER:
			return !self->markedForRemoval;
		default:
			return false;
	}
}

void HealPowerupModel_collision(compat_type(HealPowerupModel *) selfPtr, compat_type(GameObjectModel *) objectPtr, const struct collision collision, float time) {
	HealPowerupModel * self = selfPtr;
	
	self->markedForRemoval = true;
}
