/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/RoomExitModel.h"

RoomExitModel * RoomExitModel_create(DungeonRoomStateModel * stateModel, Atom name, struct rect4f bounds) {
	stemobject_create_implementation(RoomExitModel, init, stateModel, name, bounds)
}

static void sharedInit(RoomExitModel * self) {
	self->type = GAME_OBJECT_TYPE_ROOM_EXIT;
	self->dispose = RoomExitModel_dispose;
	self->copy = RoomExitModel_copy;
	self->roughBounds = RoomExitModel_roughBounds;
	self->collisionGeometry = RoomExitModel_collisionGeometry;
	self->wouldCollide = RoomExitModel_wouldCollide;
}

void RoomExitModel_init(compat_type(RoomExitModel *) selfPtr, DungeonRoomStateModel * stateModel, Atom name, struct rect4f bounds) {
	RoomExitModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->name = name;
	self->bounds = bounds;
}

void RoomExitModel_initCopy(compat_type(RoomExitModel *) selfPtr, compat_type(RoomExitModel *) originalPtr) {
	RoomExitModel * self = selfPtr;
	RoomExitModel * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->name = original->name;
	self->bounds = original->bounds;
}

void RoomExitModel_dispose(compat_type(RoomExitModel *) selfPtr) {
	RoomExitModel * self = selfPtr;
	
	StemObject_dispose(self);
}

compat_type(RoomExitModel *) RoomExitModel_copy(compat_type(RoomExitModel *) selfPtr) {
	stemobject_copy_implementation(RoomExitModel, initCopy)
}

struct rect4f RoomExitModel_roughBounds(compat_type(RoomExitModel *) selfPtr) {
	RoomExitModel * self = selfPtr;
	
	return self->bounds;
}

struct collisionGeometry RoomExitModel_collisionGeometry(compat_type(RoomExitModel *) selfPtr) {
	RoomExitModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->bounds.left;
	geometry.u.sweptAABB.current.y = self->bounds.bottom;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = self->bounds.right - self->bounds.left;
	geometry.u.sweptAABB.height = self->bounds.top - self->bounds.bottom;
	return geometry;
}

bool RoomExitModel_wouldCollide(compat_type(RoomExitModel *) selfPtr, compat_type(GameObjectModel *) objectPtr) {
	switch (((GameObjectModel *) objectPtr)->type) {
		case GAME_OBJECT_TYPE_PLAYER:
		case GAME_OBJECT_TYPE_ENEMY:
			return true;
			
		default:
			return false;
	}
}
