/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __ROOM_EXIT_MODEL_H__
#define __ROOM_EXIT_MODEL_H__

typedef struct RoomExitModel RoomExitModel;

#include "convergence/GameObjectModel.h"
#include "convergence/SharedDefinitions.h"
#include "utilities/Atom.h"

#define STAIRCASE_RADIUS 0.5f

#define RoomExitModel_structContents \
	GameObjectModel_structContents \
	\
	Atom name; \
	struct rect4f bounds;

struct RoomExitModel {
	RoomExitModel_structContents
};

RoomExitModel * RoomExitModel_create(DungeonRoomStateModel * stateModel, Atom name, struct rect4f bounds);
void RoomExitModel_init(compat_type(RoomExitModel *) selfPtr, DungeonRoomStateModel * stateModel, Atom name, struct rect4f bounds);
void RoomExitModel_initCopy(compat_type(RoomExitModel *) selfPtr, compat_type(RoomExitModel *) originalPtr);
void RoomExitModel_dispose(compat_type(RoomExitModel *) selfPtr);

compat_type(RoomExitModel *) RoomExitModel_copy(compat_type(RoomExitModel *) selfPtr);
struct rect4f RoomExitModel_roughBounds(compat_type(RoomExitModel *) selfPtr);
struct collisionGeometry RoomExitModel_collisionGeometry(compat_type(RoomExitModel *) selfPtr);
bool RoomExitModel_wouldCollide(compat_type(RoomExitModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);

#endif
