/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#include "convergence/StaircaseModel.h"

StaircaseModel * StaircaseModel_create(DungeonRoomStateModel * stateModel, Vector2 position, bool down) {
	stemobject_create_implementation(StaircaseModel, init, stateModel, position, down)
}

static void sharedInit(StaircaseModel * self) {
	self->type = GAME_OBJECT_TYPE_STAIRCASE;
	self->dispose = StaircaseModel_dispose;
	self->copy = StaircaseModel_copy;
	self->roughBounds = StaircaseModel_roughBounds;
	self->collisionGeometry = StaircaseModel_collisionGeometry;
}

void StaircaseModel_init(compat_type(StaircaseModel *) selfPtr, DungeonRoomStateModel * stateModel, Vector2 position, bool down) {
	StaircaseModel * self = selfPtr;
	
	GameObjectModel_init(self, stateModel);
	sharedInit(self);
	self->position = position;
	self->down = down;
}

void StaircaseModel_initCopy(compat_type(StaircaseModel *) selfPtr, compat_type(StaircaseModel *) originalPtr) {
	StaircaseModel * self = selfPtr;
	StaircaseModel * original = originalPtr;
	
	GameObjectModel_initCopy(self, original);
	sharedInit(self);
	self->position = original->position;
}

void StaircaseModel_dispose(compat_type(StaircaseModel *) selfPtr) {
	StaircaseModel * self = selfPtr;
	
	StemObject_dispose(self);
}

compat_type(StaircaseModel *) StaircaseModel_copy(compat_type(StaircaseModel *) selfPtr) {
	stemobject_copy_implementation(StaircaseModel, initCopy)
}

struct rect4f StaircaseModel_roughBounds(compat_type(StaircaseModel *) selfPtr) {
	StaircaseModel * self = selfPtr;
	struct rect4f bounds;
	
	bounds.left = self->position.x - STAIRCASE_RADIUS;
	bounds.right = self->position.x + STAIRCASE_RADIUS;
	bounds.bottom = self->position.y - STAIRCASE_RADIUS;
	bounds.top = self->position.y + STAIRCASE_RADIUS;
	return bounds;
}

struct collisionGeometry StaircaseModel_collisionGeometry(compat_type(StaircaseModel *) selfPtr) {
	StaircaseModel * self = selfPtr;
	struct collisionGeometry geometry;
	
	geometry.type = COLLISION_GEOMETRY_TYPE_SWEPT_AABB;
	geometry.u.sweptAABB.current.x = self->position.x - STAIRCASE_RADIUS;
	geometry.u.sweptAABB.current.y = self->position.y - STAIRCASE_RADIUS;
	geometry.u.sweptAABB.future = geometry.u.sweptAABB.current;
	geometry.u.sweptAABB.width = STAIRCASE_RADIUS * 2.0f;
	geometry.u.sweptAABB.height = STAIRCASE_RADIUS * 2.0f;
	return geometry;
}
