/*
  Copyright (c) 2011 Alex Diener
  
  This software is provided 'as-is', without any express or implied
  warranty. In no event will the authors be held liable for any damages
  arising from the use of this software.
  
  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:
  
  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
  
  Alex Diener adiener@sacredsoftware.net
*/

#ifndef __SWORD_SWING_MODEL_H__
#define __SWORD_SWING_MODEL_H__

typedef struct SwordSwingModel SwordSwingModel;

#include "convergence/GameObjectModel.h"
#include "convergence/MovingEntityModel.h"

#define SwordSwingModel_structContents \
	GameObjectModel_structContents \
	\
	int rawProgress; \
	float progress; \
	MovingEntityModel * entity; \
	Vector2 position; \
	Direction facing; \
	bool hostile; \
	int damage; \
	float knockback; \
	float minRadius; \
	float maxRadius; \
	float ratio; \
	int length;

struct SwordSwingModel {
	SwordSwingModel_structContents
};

SwordSwingModel * SwordSwingModel_create(DungeonRoomStateModel * stateModel, compat_type(MovingEntityModel *) entity, bool hostile, int damage, float knockback, float minRadius, float maxRadius, float ratio, int length);
void SwordSwingModel_init(compat_type(SwordSwingModel *) selfPtr, DungeonRoomStateModel * stateModel, compat_type(MovingEntityModel *) entity, bool hostile, int damage, float knockback, float minRadius, float maxRadius, float ratio, int length);
void SwordSwingModel_initCopy(compat_type(SwordSwingModel *) selfPtr, compat_type(SwordSwingModel *) originalPtr);
void SwordSwingModel_dispose(compat_type(SwordSwingModel *) selfPtr);

compat_type(SwordSwingModel *) SwordSwingModel_copy(compat_type(SwordSwingModel *) selfPtr);
void SwordSwingModel_update(compat_type(SwordSwingModel *) selfPtr);
bool SwordSwingModel_atRest(compat_type(SwordSwingModel *) selfPtr);
struct rect4f SwordSwingModel_roughBounds(compat_type(SwordSwingModel *) selfPtr);
struct collisionGeometry SwordSwingModel_collisionGeometry(compat_type(SwordSwingModel *) selfPtr);
bool SwordSwingModel_wouldCollide(compat_type(SwordSwingModel *) selfPtr, compat_type(GameObjectModel *) objectPtr);

#endif
